/* $Id: mach64_io.h,v 1.4 1999/06/27 16:01:15 marcus Exp $
******************************************************************************

   ATI Mach64 Series -- IO register state macros and data structures

   Copyright (C) 1997		Simon Hammett

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef	_MACH64_IO_H__
#define	_MACH64_IO_H__

#include "ATI/mach64.h"
#include <kgi/kgi.h>
/*----
 * debugging macros to check input values. isn't included if debug level = 0
 */

#if DEBUG_LEVEL > 0
#define	CHECK_OFFSET	\
	if( offset < 0 || offset > 3 )\
	{\
		printk(KERN_ERR "Mach64_IO:invalid offset for\n" );\
		printk(KERN_ERR "Mach64_IO:Mach64_Read_Register_XX.\n");\
		return 0;\
	}
#else
#define	CHECK_OFFSET
#endif

#if DEBUG_LEVEL > 0
#define	CHECK_REGISTER \
	if( reg < 0 || reg > M64Reg_Last_Register )\
	{\
		printk(KERN_ERR "Mach64_IO:invalid register address.\n" );\
		return 0;\
	}
#else
#define	CHECK_REGISTER
#endif


/*
 * enumerate the Mach64 registers.
 */

enum	{
		Crtc_H_Total,
		Crtc_H_Sync,
		Crtc_V_Total,
		Crtc_V_Sync,
		Crtc_Vline,
		Crtc_Display_Start,
		Crtc_Int_Cntl,
		Crtc_Gen_Cntl,
		Over_Scan_Colour,
		Over_Scan_Width_LR,
		Over_Scan_Height_TB,
		Cursor_Colour_0,
		Cursor_Colour_1,
		Cursor_Offset,
		Cursor_XY,
		Cursor_Hotspot_XY,
		Scratch_Reg0,
		Test_Reg0,
		Test_Reg2,
		Test_Reg4,
		Test_Reg5,
		Test_Reg7,
		Scratch_Reg1,
		Test_Reg1,
		Test_Reg3,
		Test_Reg6,
		Clock_Cntl,
		Bus_Cntl,
		Mem_Cntl,
		Mem_VGA_Write_Window,
		Mem_VGA_Read_Window,
		Dac_Regs,
		Dac_Cntl,
		Gen_Test_Cntl,
		Config_Chip_ID,
		Config_Stat0,
		Config_Stat1,
		Config_Cntl,
		Last_Register
	};


/*----
 * unfortunately the mach64s io regs go out of step with the mmio regs
 * so we can't just calculate the io reg from the mmio register. :(
 *
 */


#ifdef _MACH64_IO_C__ 

	int	Mach64_MMIO_Registers[]=
		{
			0x000,		/* Crtc_H_Total */
			0x004,		/* Crtc_H_Sync */
			0x008,		/* Crtc_V_Total */
			0x00c,		/* Crtc_V_Sync */
			0x010,		/* Crtc_Vline */
			0x014,		/* Crtc_Display_Start */
			0x018,		/* Crtc_Int_Cntl */
			0x01c,		/* Crtc_Gen_Cntl */
			0x040,		/* Over_Scan_Colour */
			0x044,		/* Over_Scan_Width_LR */
			0x048,		/* Over_Scan_Height_TB */
			0x060,		/* Cursor_Colour_0 */
			0x064,		/* Cursor_Colour_1 */
			0x068,		/* Cursor_Offset */
			0x06c,		/* Cursor_XY */
			0x070,		/* Cursor_Hotspot_XY */
			0x080,		/* Scratch_Reg0 */
			0x080,		/* Test_Reg0 */
			0x080,		/* Test_Reg2 */
			0x080,		/* Test_Reg4 */
			0x080,		/* Test_Reg5 */
			0x080,		/* Test_Reg7 */
			0x084,		/* Scratch_Reg1 */
			0x084,		/* Test_Reg1 */
			0x084,		/* Test_Reg3 */
			0x084,		/* Test_Reg6 */
			0x090,		/* Clock_Cntl */
			0x0a0,		/* Bus_Cntl */
			0x0b0,		/* Mem_Cntl */
			0x0b4,		/* Mem_VGA_Write_Window */
			0x0b8,		/* Mem_VGA_Read_Window */
			0x0c0,		/* Dac_Regs */
			0x0c4,		/* Dac_Cntl */
			0x0d0,		/* Gen_Test_Cntl */
			0x0e0,		/* Config_Chip_ID */
			0x0e4,		/* Config_Stat0 */
			0x0e8,		/* Config_Stat1 */
			0x080		/* Config_Cntl ( note mmio -> scratch0 ) */
					/* as config_cntl is not mmio mapped */
		};
  int     Mach64_Registers[]=
                {
                        0x02ec,          /* Crtc_H_Total */
                        0x06ec,          /* Crtc_H_Sync */
                        0x0aec,          /* Crtc_V_Total */
                        0x0eec,          /* Crtc_V_Sync */
                        0x12ec,          /* Crtc_Vline */
                        0x16ec,          /* Crtc_Display_Start */
                        0x1aec,          /* Crtc_Int_Cntl */
                        0x1eec,          /* Crtc_Gen_Cntl */
                        0x22ec,          /* Over_Scan_Colour */
                        0x26ec,          /* Over_Scan_Width_LR */
                        0x2aec,          /* Over_Scan_Height_TB */
                        0x2eec,          /* Cursor_Colour_0 */
                        0x32ec,          /* Cursor_Colour_1 */
                        0x36ec,          /* Cursor_Offset */
                        0x3aec,          /* Cursor_XY */
                        0x3eec,          /* Cursor_Hotspot_XY */
                        0x42ec,          /* Scratch_Reg0 */
                        0x42ec,          /* Test_Reg0 */
                        0x42ec,          /* Test_Reg2 */
                        0x42ec,          /* Test_Reg4 */
                        0x42ec,          /* Test_Reg5 */
                        0x42ec,          /* Test_Reg7 */
                        0x46ec,          /* Scratch_Reg1 */
                        0x46ec,          /* Test_Reg1 */
                        0x46ec,          /* Test_Reg3 */
                        0x46ec,          /* Test_Reg6 */
                        0x4aec,          /* Clock_Cntl */
                        0x4eec,          /* Bus_Cntl */
                        0x52ec,          /* Mem_Cntl */
                        0x56ec,          /* Mem_VGA_Write_Window */
                        0x5aec,          /* Mem_VGA_Read_Window */
                        0x5eec,          /* Dac_Regs */
                        0x62ec,          /* Dac_Cntl */
                        0x66ec,          /* Gen_Test_Cntl */
                        0x6eec,          /* Config_Chip_ID */
                        0x72ec,          /* Config_Stat0 */
                        0x76ec,          /* Config_Stat1 */
                        0x6aec           /* Config_Cntl ( note mmio -> scratch 0 ) */
                                         /* as config_cntl is not mmio mapped */
                }; 

#define PLL_WR_EN	0x02

static kgiu16 vga_dac_ports[4] = {
	0x3C8, 0x3C9, 0x3C6, 0x3C7	};

#else
    extern int Mach64_Registers[];
    extern int Mach64_MMIO_Registers[];
#endif

/*
 *	note *Register and *Register_32 are the same,
 *	included *32 for completeness.
 */


inline	int	mach64_out8( int reg, int offset, 
				kgiu8 value, struct kgi_display* dpy );
/* inline	int	mach64_out16( int reg, kgiu16 value ); */
inline	int	mach64_out32( int reg, kgiu32 value , struct kgi_display* dpy);

inline	kgiu8	mach64_in8( int reg, int offset, struct kgi_display* dpy );
/* inline	kgiu16	mach64_in16( int reg, int offset ); */
inline	kgiu32	mach64_in32( int reg , struct kgi_display* dpy);

inline void mach64_set_PLL( int offset, kgiu8 value , struct cardInfo *card);

#endif
